<?php

namespace IGD;

defined( 'ABSPATH' ) || exit;
class Rest_Api_Controller
{
    /** @var null */
    private static  $instance = null ;
    private  $namespace = 'igd/v1' ;
    /**
     * Rest_Api_Controller constructor.
     *
     */
    public function __construct()
    {
        add_action( 'rest_api_init', [ $this, 'register_api' ] );
    }
    
    /**
     * Register rest API
     *
     * @since 1.0.0
     */
    public function register_api()
    {
        // Move File/ Folder
        register_rest_route( $this->namespace, '/move-file/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'move_file' ),
            'permission_callback' => '__return_true',
        ) ) );
        //handle rename
        register_rest_route( $this->namespace, '/rename/', array( array(
            'methods'             => \WP_REST_Server::EDITABLE,
            'callback'            => array( $this, 'handle_rename' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Copy file/ folder
        register_rest_route( $this->namespace, '/copy/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'handle_copy' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Create a new folder
        register_rest_route( $this->namespace, '/new-folder/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'new_folder' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Switch Account
        register_rest_route( $this->namespace, '/switch-account/', array( array(
            'methods'             => \WP_REST_Server::EDITABLE,
            'callback'            => array( $this, 'switch_account' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Delete Account
        register_rest_route( $this->namespace, '/delete-account/', array( array(
            'methods'             => \WP_REST_Server::EDITABLE,
            'callback'            => array( $this, 'delete_account' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Set Permission
        register_rest_route( $this->namespace, '/set-permission/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'set_permission' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Save Settings
        register_rest_route( $this->namespace, '/save-settings/', array( array(
            'methods'             => \WP_REST_Server::EDITABLE,
            'callback'            => array( $this, 'save_settings' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Update user folders
        register_rest_route( $this->namespace, '/update-user-folders/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'update_user_folders' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get users data
        register_rest_route( $this->namespace, '/get-users-data/', array( array(
            'methods'             => 'POST',
            'callback'            => array( $this, 'get_users_data' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Update Shortcode
        register_rest_route( $this->namespace, '/update-shortcode/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'update_shortcode' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Duplicate Shortcode
        register_rest_route( $this->namespace, '/duplicate-shortcode/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'duplicate_shortcode' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get Shortcode Content
        register_rest_route( $this->namespace, '/get-shortcode-content/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'get_shortcode_content' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get Files by folder id
        register_rest_route( $this->namespace, '/get-files/', array( array(
            'methods'             => 'POST',
            'callback'            => array( $this, 'get_files' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get File by file id
        register_rest_route( $this->namespace, '/get-file/', array( array(
            'methods'             => 'POST',
            'callback'            => array( $this, 'get_file' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Search Files
        register_rest_route( $this->namespace, '/search-files/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'search_files' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get Embed Content
        register_rest_route( $this->namespace, '/get-embed-content/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'get_embed_content' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Delete Files
        register_rest_route( $this->namespace, '/delete-files/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'delete_files' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Create Doc
        register_rest_route( $this->namespace, '/create-doc/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'create_doc' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get export data
        register_rest_route( $this->namespace, '/export-data/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'get_export_data' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Import Data
        register_rest_route( $this->namespace, '/import-data/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'import_data' ),
            'permission_callback' => '__return_true',
        ) ) );
        // File Uploaded
        register_rest_route( $this->namespace, '/file-uploaded/', array( array(
            'methods'             => \WP_REST_Server::CREATABLE,
            'callback'            => array( $this, 'upload_post_process' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Get Shortcodes
        register_rest_route( $this->namespace, '/get-shortcodes/', array( array(
            'methods'             => 'POST',
            'callback'            => array( $this, 'get_shortcodes' ),
            'permission_callback' => '__return_true',
        ) ) );
        // Update Description
        register_rest_route( $this->namespace, '/update-description/', array( array(
            'methods'             => 'POST',
            'callback'            => array( $this, 'update_description' ),
            'permission_callback' => '__return_true',
        ) ) );
    }
    
    public function get_file( $request )
    {
        $params = $request->get_json_params();
        $file_id = ( !empty($params['id']) ? $params['id'] : '' );
        $account_id = ( !empty($params['accountId']) ? $params['accountId'] : '' );
        $file = App::instance( $account_id )->get_file_by_id( $file_id );
        return rest_ensure_response( $file );
    }
    
    public function update_description( $request )
    {
        $data = $request->get_json_params();
        $id = ( !empty($data['id']) ? sanitize_text_field( $data['id'] ) : '' );
        $account_id = ( !empty($data['accountId']) ? sanitize_text_field( $data['accountId'] ) : '' );
        $description = ( !empty($data['description']) ? sanitize_text_field( $data['description'] ) : '' );
        $update_file = App::instance( $account_id )->update_description( $id, $description );
        return rest_ensure_response( $update_file );
    }
    
    public function get_shortcodes( $request )
    {
        $params = $request->get_json_params();
        $page = ( !empty($params['page']) ? $params['page'] : 1 );
        $per_page = ( !empty($params['per_page']) ? $params['per_page'] : 10 );
        $args = [];
        $args['offset'] = 10 * ($page - 1);
        $args['limit'] = $per_page;
        $shortcodes = Shortcode_Builder::instance()->get_shortcodes( $args );
        $data = [];
        $formatted = [];
        if ( !empty($shortcodes) ) {
            foreach ( $shortcodes as $shortcode ) {
                $shortcode->config = maybe_unserialize( $shortcode->config );
                $shortcode->locations = ( !empty($shortcode->locations) ? array_values( maybe_unserialize( $shortcode->locations ) ) : [] );
                $formatted[] = $shortcode;
            }
        }
        $data['shortcodes'] = $formatted;
        $data['total'] = Shortcode_Builder::instance()->get_shortcodes_count();
        return rest_ensure_response( $data );
    }
    
    public function upload_post_process( $request )
    {
        $data = $request->get_json_params();
        $file = ( !empty($data['file']) ? $data['file'] : [] );
        $account_id = ( !empty($file['accountId']) ? sanitize_text_field( $file['accountId'] ) : '' );
        Uploader::instance( $account_id )->upload_post_process( $file );
        return rest_ensure_response( array(
            'success' => true,
        ) );
    }
    
    public function import_data( $request )
    {
        $settings = $request->get_param( 'settings' );
        $shortcodes = $request->get_param( 'shortcodes' );
        $user_files = $request->get_param( 'user_files' );
        $events = $request->get_param( 'events' );
        if ( !empty($settings) ) {
            update_option( 'igd_settings', $settings );
        }
        
        if ( !empty($shortcodes) ) {
            $shortcode_builder = Shortcode_Builder::instance();
            $shortcode_builder->delete_shortcode();
            foreach ( $shortcodes as $shortcode ) {
                $shortcode_builder->update_shortcode( $shortcode, true );
            }
        }
        
        if ( !empty($user_files) ) {
            foreach ( $user_files as $user_id => $files ) {
                update_user_option( $user_id, 'folders', $files );
            }
        }
        return rest_ensure_response( array(
            'status' => 'success',
        ) );
    }
    
    public function get_export_data( $request )
    {
        $type = $request->get_param( 'type' );
        $type = ( !empty($type) ? $type : 'all' );
        $export_data = array();
        // Settings
        if ( 'all' == $type || 'settings' == $type ) {
            $export_data['settings'] = igd_get_settings();
        }
        // Shortcodes
        if ( 'all' == $type || 'shortcodes' == $type ) {
            $export_data['shortcodes'] = Shortcode_Builder::instance()->get_shortcodes();
        }
        // User Private Files
        
        if ( 'all' == $type || 'user_files' == $type ) {
            $user_files = array();
            $users = get_users();
            foreach ( $users as $user ) {
                $folders = get_user_option( 'folders', $user->ID );
                $user_files[$user->ID] = ( !empty($folders) ? $folders : array() );
            }
            $export_data['user_files'] = $user_files;
        }
        
        return rest_ensure_response( $export_data );
    }
    
    public function create_doc( $request )
    {
        $posted = $request->get_json_params();
        $name = $posted['name'];
        $type = $posted['type'];
        $folder_id = ( !empty($posted['folder_id']) ? $posted['folder_id'] : Account::instance()->get_active_account()['root_id'] );
        $account_id = ( !empty($posted['account_id']) ? $posted['account_id'] : Account::instance()->get_active_account()['id'] );
        $mime_type = 'application/vnd.google-apps.document';
        
        if ( $type == 'sheet' ) {
            $mime_type = 'application/vnd.google-apps.spreadsheet';
        } elseif ( $type == 'slide' ) {
            $mime_type = 'application/vnd.google-apps.presentation';
        }
        
        try {
            $item = App::instance( $account_id )->getService()->files->create( new \IGDGoogle_Service_Drive_DriveFile( [
                'name'     => $name,
                'mimeType' => $mime_type,
                'parents'  => [ $folder_id ],
            ] ), [
                'fields' => '*',
            ] );
            // add new folder to cache
            $file = igd_file_map( $item, $account_id );
            Files::add_file( $file );
            // Insert log
            do_action(
                'igd_insert_log',
                'create',
                $file['id'],
                $account_id
            );
            return rest_ensure_response( $file );
        } catch ( \Exception $e ) {
            return rest_ensure_response( array(
                'error' => $e->getMessage(),
            ) );
        }
    }
    
    public function delete_files( $request )
    {
        $posted = $request->get_json_params();
        $file_ids = ( !empty($posted['file_ids']) ? $posted['file_ids'] : [] );
        $account_id = ( !empty($posted['account_id']) ? $posted['account_id'] : '' );
        //send email notification
        if ( igd_get_settings( 'deleteNotifications', true ) ) {
            do_action(
                'igd_send_notification',
                'delete',
                $file_ids,
                $account_id
            );
        }
        wp_send_json_success( App::instance( $account_id )->delete( $file_ids, $account_id ) );
    }
    
    public function get_embed_content( $request )
    {
        $posted = $request->get_json_params();
        $content = igd_get_embed_content( $posted );
        wp_send_json_success( $content );
    }
    
    public function get_files( $request )
    {
        $active_account = Account::instance()->get_active_account();
        if ( empty($active_account) ) {
            wp_send_json_error( __( 'No active account found', 'integrate-google-drive' ) );
        }
        $args = [
            'folder'      => [
            'id'         => $active_account['root_id'],
            'accountId'  => $active_account['id'],
            'pageNumber' => 1,
        ],
            'sort'        => [
            'sortBy'        => 'name',
            'sortDirection' => 'asc',
        ],
            'from_server' => false,
            'filters'     => [],
        ];
        // Merge request params
        
        if ( !is_object( $request ) ) {
            $args = wp_parse_args( $request, $args );
        } else {
            $args = wp_parse_args( $request->get_json_params(), $args );
        }
        
        $account_id = ( !empty($folder['accountId']) ? $folder['accountId'] : $active_account['id'] );
        $file_numbers = ( !empty($args['fileNumbers']) ? intval( $args['fileNumbers'] ) : 0 );
        $refresh = !empty($args['refresh']);
        $folder = $args['folder'];
        
        if ( !empty($args['from_server']) ) {
            $transient = get_transient( 'igd_latest_fetch_' . $folder['id'] );
            
            if ( $transient ) {
                $args['from_server'] = false;
            } else {
                set_transient( 'igd_latest_fetch_' . $folder['id'], true, 60 * MINUTE_IN_SECONDS );
            }
        
        }
        
        // Check if shortcut folder
        if ( !empty($folder['shortcutDetails']) ) {
            $args['folder']['id'] = $folder['shortcutDetails']['targetId'];
        }
        // Reset cache and get new files
        
        if ( $refresh ) {
            $refresh_args = $args;
            $refresh_args['folder']['pageNumber'] = 1;
            $refresh_args['from_server'] = true;
            $data = App::instance( $account_id )->get_files( $refresh_args );
        } else {
            $data = App::instance( $account_id )->get_files( $args );
        }
        
        // Handle maximum file to show
        if ( $file_numbers > 0 && count( $data['files'] ) > $file_numbers ) {
            $data['files'] = array_slice( $data['files'], 0, $file_numbers );
        }
        if ( !is_object( $request ) ) {
            return $data;
        }
        if ( !empty($data['error']) ) {
            return rest_ensure_response( $data );
        }
        if ( empty($folder['pageNumber']) || $folder['pageNumber'] == 1 ) {
            $data['breadcrumbs'] = igd_get_breadcrumb( $folder );
        }
        return rest_ensure_response( $data );
    }
    
    public function search_files( $request )
    {
        
        if ( !is_object( $request ) ) {
            $data = $request;
        } else {
            $data = $request->get_json_params();
        }
        
        $folders = ( !empty($data['folders']) ? $data['folders'] : [] );
        $keyword = ( !empty($data['keyword']) ? $data['keyword'] : '' );
        $account_id = ( !empty($data['accountId']) ? $data['accountId'] : Account::instance()->get_active_account()['id'] );
        $sort = ( !empty($data['sort']) ? $data['sort'] : [
            'sortBy'        => 'name',
            'sortDirection' => 'asc',
        ] );
        $full_text_search = ( isset( $data['fullTextSearch'] ) ? $data['fullTextSearch'] : true );
        $file_numbers = ( !empty($data['fileNumbers']) ? $data['fileNumbers'] : 1000 );
        $filters = ( !empty($data['filters']) ? $data['filters'] : [] );
        $files = App::instance( $account_id )->get_search_files(
            $keyword,
            $folders,
            $sort,
            $full_text_search
        );
        if ( !empty($files['error']) ) {
            return rest_ensure_response( $files );
        }
        // Filter files
        
        if ( !empty($files) && igd_should_filter_files( $filters ) ) {
            $filter_files = [];
            foreach ( $files as $item ) {
                $is_dir = igd_is_dir( $item );
                if ( !$is_dir && !$filters['showFiles'] ) {
                    continue;
                }
                if ( $is_dir && !$filters['showFolders'] ) {
                    continue;
                }
                $extension = ( !empty($item['extension']) ? $item['extension'] : '' );
                $name = ( !empty($item['name']) ? $item['name'] : '' );
                if ( !igd_should_allow(
                    $filters,
                    $extension,
                    $name,
                    $is_dir
                ) ) {
                    continue;
                }
                $filter_files[] = $item;
            }
            $files = $filter_files;
        }
        
        // Handle maximum file to show
        if ( $file_numbers > 0 && count( $files ) > $file_numbers ) {
            $files = array_slice( $files, 0, $file_numbers );
        }
        if ( !is_object( $request ) ) {
            return $files;
        }
        $data = [
            'files' => $files,
        ];
        return rest_ensure_response( $data );
    }
    
    public function get_shortcode_content( $request )
    {
        $data = $request->get_json_params();
        $html = Shortcode::instance()->render_shortcode( [], $data );
        wp_send_json_success( $html );
    }
    
    public function update_shortcode( $request )
    {
        $shortcode_data = $request->get_json_params();
        $id = Shortcode_Builder::instance()->update_shortcode( $shortcode_data );
        $data = [
            'id'         => $id,
            'config'     => $shortcode_data,
            'title'      => $shortcode_data['title'],
            'status'     => $shortcode_data['status'],
            'created_at' => ( !empty($shortcode_data['created_at']) ? $shortcode_data['created_at'] : date( 'Y-m-d H:i:s', time() ) ),
        ];
        wp_send_json_success( $data );
    }
    
    public function duplicate_shortcode( $request )
    {
        $ids = $request->get_param( 'ids' );
        $data = [];
        if ( !empty($ids) ) {
            foreach ( $ids as $id ) {
                $data[] = Shortcode_Builder::instance()->duplicate_shortcode( $id );
            }
        }
        return rest_ensure_response( $data );
    }
    
    /**
     * Get users data
     *
     * @param $request
     *
     * @return void
     */
    public function get_users_data( $request )
    {
        $data = json_decode( $request->get_body(), 1 );
        $args = [];
        
        if ( !empty($data) ) {
            $search = ( !empty($data['search']) ? $data['search'] : '' );
            $role = ( !empty($data['role']) ? $data['role'] : '' );
            $page = ( !empty($data['page']) ? $data['page'] : 1 );
            $offset = 10 * ($page - 1);
            $args = [
                'number' => 10,
                'role'   => ( 'all' != $role ? $role : '' ),
                'offset' => $offset,
                'search' => ( !empty($search) ? "*{$search}*" : '' ),
            ];
        }
        
        $user_data = Private_Folders::instance()->get_user_data( $args );
        wp_send_json_success( $user_data );
    }
    
    /**
     * Update user folders
     *
     * @param $request
     *
     * @return void
     */
    public function update_user_folders( $request )
    {
        $data = json_decode( $request->get_body(), 1 );
        $id = $data['id'];
        $folders = $data['folders'];
        update_user_option( $id, 'folders', $folders );
        wp_send_json_success();
    }
    
    /**
     * @param $request
     *
     * @return void
     */
    public function save_settings( $request )
    {
        $data = json_decode( $request->get_body(), 1 );
        update_option( 'igd_settings', $data );
        wp_send_json_success();
    }
    
    public function new_folder( $request )
    {
        $data = json_decode( $request->get_body(), 1 );
        $folder_name = sanitize_text_field( $data['name'] );
        $parent_folder = $data['parent'];
        $parent_folder = ( !empty($parent_folder['id']) ? $parent_folder['id'] : $parent_folder );
        $account_id = ( !empty($parent_folder['accountId']) ? $parent_folder['accountId'] : null );
        $new_folder = App::instance( $account_id )->new_folder( $folder_name, $parent_folder );
        return rest_ensure_response( $new_folder );
    }
    
    public function set_permission( $request )
    {
        $requested = json_decode( $request->get_body(), 1 );
        $files = ( !empty($requested['files']) ? $requested['files'] : [] );
        if ( !empty($files) ) {
            foreach ( $files as $file ) {
                $account_id = ( !empty($file['accountId']) ? $file['accountId'] : '' );
                $permissions = Permissions::instance( $account_id );
                if ( !$permissions->has_permission( $file ) ) {
                    $permissions->set_permission( $file );
                }
            }
        }
        wp_send_json_success();
    }
    
    // Delete account
    public function delete_account( $request )
    {
        $data = json_decode( $request->get_body() );
        $id = $data->id;
        Account::instance()->delete_account( $id );
        wp_send_json_success();
    }
    
    // Switch account
    public function switch_account( $request )
    {
        $data = $request->get_json_params();
        if ( empty($data['id']) ) {
            return rest_ensure_response( new \WP_Error( 'error', 'Account id is required', array(
                'status' => 400,
            ) ) );
        }
        Account::instance()->set_active_account( $data['id'] );
        wp_send_json_success();
    }
    
    public function move_file( $request )
    {
        $posted = json_decode( $request->get_body(), 1 );
        $file_ids = ( !empty($posted['file_ids']) ? $posted['file_ids'] : '' );
        $folder_id = ( !empty($posted['folder_id']) ? sanitize_text_field( $posted['folder_id'] ) : '' );
        $account_id = ( !empty($posted['account_id']) ? $posted['account_id'] : null );
        wp_send_json_success( App::instance( $account_id )->move_file( $file_ids, $folder_id ) );
    }
    
    public function handle_rename( $request )
    {
        $data = json_decode( $request->get_body(), 1 );
        $name = sanitize_text_field( $data['name'] );
        $file_id = $data['id'];
        $account_id = ( !empty($data['accountId']) ? $data['accountId'] : null );
        wp_send_json_success( App::instance( $account_id )->rename( $name, $file_id ) );
    }
    
    public function handle_copy( $request )
    {
        $data = $request->get_json_params();
        $files = ( !empty($data['files']) ? $data['files'] : [] );
        $folder_id = ( !empty($data['folder_id']) ? $data['folder_id'] : '' );
        $account_id = ( !empty($files[0]['accountId']) ? $files[0]['accountId'] : null );
        $copied_files = App::instance( $account_id )->copy( $files, $folder_id );
        return rest_ensure_response( $copied_files );
    }
    
    /**
     * @return Rest_Api_Controller|null
     */
    public static function instance()
    {
        if ( is_null( self::$instance ) ) {
            self::$instance = new self();
        }
        return self::$instance;
    }

}
Rest_Api_Controller::instance();